@@ -17,7 +17,7 @@
  */
 #ifndef lint
 static char *rcsid =
-"$Id: avail.c,v 1.47 1996/08/18 15:27:16 stolcke Exp $ ICSI (Berkeley)";
+"$Id: avail.c,v 1.50 2001/02/17 01:09:58 stolcke Exp $ ICSI (Berkeley)";
 #endif /* not lint */
 
 #include    <signal.h>
@@ -39,6 +39,7 @@ int			npriLevel = DEF_NPRI_LEVEL;
 int			checkUser = DEF_CHECK_USER;
 static int		evictDelay = DEF_EVICT_DELAY;
 int			localJobs = DEF_LOCAL_JOBS;
+int			maxExclusives = 1;
 
 static Rpc_Event  	availEvent;	/* Event for checking availability */
 static struct timeval	availInterval;	/* Interval at which checks should
@@ -50,6 +51,7 @@ int	    	    	avail_Bias; 	/* Bias for rating calculation */
 double			avail_LoadBias = 0.0; /* Bias for load correction */
 static Rpc_Event	decayEvent;	/* Event for decaying load bias */
 
+int			avail_Exclusive;/* No of 'exclusive' jobs on server */
 static Rpc_Event	evictEvent;	/* Event for evicting imported jobs */
 static struct timeval	evictInterval;	/* Delay till eviction */
 
@@ -521,37 +523,11 @@ Avail_Local(what, ratingPtr)
      */
     what &= availCheck;
 
-    /*
-     * If a server host was specified, check that it's alive, to prevent
-     * hanging on imports. For simplicity, we assume the server also runs
-     * customs, so we ping it.
-     */
-    if (serverAddr.sin_addr.s_addr != 0) {
-	int rstat;
-
-	rstat = Rpc_Call(udpSocket, &serverAddr,
-			 (Rpc_Proc)CUSTOMS_PING,
-			 0, (Rpc_Opaque)0, 0, (Rpc_Opaque)0,
-			 CUSTOMSINT_NRETRY, &retryTimeOut);
-	if (rstat != RPC_SUCCESS) {
-	    if (verbose) {
-		xlog (XLOG_DEBUG,
-			"Avail_Local: server ping: %s",
-			Rpc_ErrorMessage(rstat));
-	    }
-	    /*
-	     * Return the server address, so reginfo can be more
-	     * informative. Note that the rating is byte-swapped, so
-	     * the address needs to be transmitted in host order.
-	     */
-	    *ratingPtr = ntohl(serverAddr.sin_addr.s_addr);
-	    return AVAIL_SERVER;
-	}
-    }
-
     /*
      * If a minimum idle time was specified, check to make sure the
      * keyboard idle time exceeds that.
+     * NOTE: It is important to check for idleness before the other 
+     * criteria, because eviction processing depends on the result.
      */
     if ((what & AVAIL_IDLE) && minIdle) {
 	long idleTime = OS_Idle(idleCrit);
@@ -590,6 +566,49 @@ Avail_Local(what, ratingPtr)
 	evictEvent = (Rpc_Event)0;
     }
 
+    /*
+     * Check if host is in exclusive use (client has requested that it be
+     * the sole user of this machine).
+     */
+    if (avail_Exclusive >= maxExclusives) {
+	if (verbose) {
+	    xlog (XLOG_DEBUG, "Avail_Local: in exclusive use");
+	}
+	/*
+	 * Return number of exclusive jobs
+	 */
+	*ratingPtr = avail_Exclusive;
+	return AVAIL_EXCLUSIVE;
+    }
+
+    /*
+     * If a server host was specified, check that it's alive, to prevent
+     * hanging on imports. For simplicity, we assume the server also runs
+     * customs, so we ping it.
+     */
+    if (serverAddr.sin_addr.s_addr != 0) {
+	int rstat;
+
+	rstat = Rpc_Call(udpSocket, &serverAddr,
+			 (Rpc_Proc)CUSTOMS_PING,
+			 0, (Rpc_Opaque)0, 0, (Rpc_Opaque)0,
+			 CUSTOMSINT_NRETRY, &retryTimeOut);
+	if (rstat != RPC_SUCCESS) {
+	    if (verbose) {
+		xlog (XLOG_DEBUG,
+			"Avail_Local: server ping: %s",
+			Rpc_ErrorMessage(rstat));
+	    }
+	    /*
+	     * Return the server address, so reginfo can be more
+	     * informative. Note that the rating is byte-swapped, so
+	     * the address needs to be transmitted in host order.
+	     */
+	    *ratingPtr = ntohl(serverAddr.sin_addr.s_addr);
+	    return AVAIL_SERVER;
+	}
+    }
+
     /*
      * Check that we're not exceeding our importquota for maximum number
      * of simultaneous jobs.
