@@ -46,7 +46,7 @@
  */
 #ifndef lint
 static char *rcsid =
-"$Id: customs.c,v 1.81 1996/08/18 15:27:16 stolcke Exp $ ICSI (Berkeley)";
+"$Id: customs.c,v 1.83 2001/02/17 01:09:58 stolcke Exp $ ICSI (Berkeley)";
 #endif /* not lint */
 
 #include    <stdio.h>
@@ -892,6 +892,7 @@ Usage ()
     printf ("\t-load <loadavg>  Set maximum load average\n");
     printf ("\t-swap <percent>  Set minimum free swap space\n");
     printf ("\t-jobs <num>      Set maximum imported jobs\n");
+    printf ("\t-exclusives <num>Set maximum exclusive jobs\n");
     printf ("\t-proc <num>      Set minimum free processes\n");
     printf ("\t-cpu <time>      Set cpu time limit for imported jobs\n");
     printf ("\t-memory <kbytes> Set memory use limit for imported jobs\n");
@@ -1036,6 +1037,16 @@ main (argc, argv)
 		Usage();
 		/*NOTREACHED*/
 	    }
+	} else if (strcmp (*argv, "-exclusives") == 0) {
+	    if (argc > 1) {
+		maxExclusives = atoi(argv[1]);
+		argc--;
+		argv++;
+	    } else {
+		printf("-exclusives needs a number of jobs as an argument\n");
+		Usage();
+		/*NOTREACHED*/
+	    }
 	} else if (strcmp (*argv, "-proc") == 0) {
 	    if (argc > 1) {
 		criteria.procs = atoi(argv[1]);
@@ -1292,7 +1303,7 @@ main (argc, argv)
 	xlog_set("customs", stdout);
 #endif /* LOG_FACILITY */
     } else {
-	xlog_set("customs", stdout, LOG_FACILITY);
+	xlog_set("customs", stdout);
     }
 
     /*
