@@ -16,7 +16,7 @@
  */
 #ifndef lint
 static char *rcsid =
-"$Id: import.c,v 1.80 1999/01/23 06:33:11 stolcke Exp $ ICSI (Berkeley)";
+"$Id: import.c,v 1.83 2001/02/17 16:39:50 stolcke Exp $ ICSI (Berkeley)";
 #endif /* not lint */
 
 #include    <sys/types.h>
@@ -69,6 +69,7 @@ typedef struct {
     ExportPermit  permit;   	/* Actual permit from MCA */
     Rpc_Event	  expire;   	/* Expiration timer. If this goes off,
 				 * the permit gets nuked */
+    int		  flags;	/* EXPORT_* flags */
 } ImportPermit;
 
 /*
@@ -469,6 +470,16 @@ ImportCheckAll()
 		}
 	    }
 
+	    /*
+	     * If process was exclusive user we are now again free for
+	     * non-exclusive use.
+	     */
+	    if (procPtr->permit->flags & EXPORT_EXCLUSIVE &&
+		avail_Exclusive > 0)
+	    {
+		avail_Exclusive --;
+	    }
+
 	    (void)Lst_Remove(imports, ln);
 	    avail_LoadBias -= LOADBIAS;
 	    Avail_Send();
@@ -506,6 +517,15 @@ ImportExpire(ln)
 
     permit = (ImportPermit *)Lst_Datum(ln);
     Rpc_EventDelete(permit->expire);
+
+    /*
+     * If process was exclusive user we are now again free for
+     * non-exclusive use.
+     */
+    if (permit->flags & EXPORT_EXCLUSIVE && avail_Exclusive > 0) {
+	avail_Exclusive --;
+    }
+
     Lst_Remove(permits, ln);
     free((char *)permit);
     avail_LoadBias -= LOADBIAS;
@@ -579,21 +599,22 @@ ImportCheckUser(uid)
  */
 /*ARGSUSED*/
 static void
-ImportAllocated(from, msg, len, permit)
+ImportAllocated(from, msg, len, data)
     struct sockaddr_in	*from;
     Rpc_Message	  	msg;
     int	    	  	len;
-    ExportPermit  	*permit;
+    AllocData  		*data;
 {
     ImportPermit  	*newPermit;
     AllocReply	    	reply;
 
-    if (len != sizeof(ExportPermit)) {
+    if (len != sizeof(AllocData)) {
 	Rpc_Error (msg, RPC_BADARGS);
     } else if ((from->sin_port == htons(udpPort)) &&
 	       ((from->sin_addr.s_addr == masterAddr.sin_addr.s_addr) ||
 		(amMaster && Local(from))))
     {
+	ExportPermit *permit = &data->permit;
 	int uid = (short)((permit->id >> 16) & 0xffff);	/* user id recovered */
 
 	if ((uid != -1) && !ImportCheckUser(uid)) {
@@ -607,6 +628,21 @@ ImportAllocated(from, msg, len, permit)
 	    Rpc_Error(msg, RPC_ACCESS);
 	    return;
 	}
+
+	/*
+	 * Check that we're not accepting exclusive jobs while machine is 
+	 * already in use.  This happens when the current jobs are not
+	 * (all) exclusive (so the machine appears available to the master),
+	 * but the incoming job has requested exclusive use.
+	 */
+	if (data->flags & EXPORT_EXCLUSIVE && Import_NJobs() >= maxExclusives) {
+	    if (verbose) {
+		xlog (XLOG_WARNING, "Exclusive allocation denied");
+	    }
+	    Rpc_Error(msg, RPC_ACCESS);
+	    return;
+	}
+
 	if (permit->addr.s_addr == CUSTOMS_FROMADDR) {
 	    permit->addr = from->sin_addr;
 	}
@@ -624,6 +660,7 @@ ImportAllocated(from, msg, len, permit)
 	 */
 	newPermit = (ImportPermit *)emalloc (sizeof (ImportPermit));
 	newPermit->permit = *permit;
+	newPermit->flags = data->flags;
 	Lst_AtEnd (permits, (ClientData)newPermit);
 	newPermit->expire =
 	    Rpc_EventCreate((uid == -1 ? &expirationNow : &expirationDate),
@@ -636,6 +673,13 @@ ImportAllocated(from, msg, len, permit)
 	 */
 	avail_LoadBias += LOADBIAS;
 
+	/*
+	 * Note exclusive use has started if requested by client
+	 */
+	if (data->flags & EXPORT_EXCLUSIVE) {
+	    avail_Exclusive ++;
+	}
+
 	reply.avail = Avail_Local(AVAIL_EVERYTHING, &reply.rating);
 	Rpc_Return(msg, sizeof(reply), (Rpc_Opaque)&reply);
     } else {
@@ -1712,7 +1756,7 @@ Import_Init()
     Rpc_ServerCreate(tcpSocket, (Rpc_Proc)CUSTOMS_IMPORT, ImportProcess,
 		     Swap_WayBill, Rpc_SwapNull, (Rpc_Opaque)0);
     Rpc_ServerCreate(udpSocket, (Rpc_Proc)CUSTOMS_ALLOC, ImportAllocated,
-		     Swap_ExportPermit, Swap_AllocReply, (Rpc_Opaque)0);
+		     Swap_AllocData, Swap_AllocReply, (Rpc_Opaque)0);
     Rpc_ServerCreate(udpSocket, (Rpc_Proc)CUSTOMS_KILL, ImportHandleKill,
 		     Swap_Kill, Rpc_SwapNull, (Rpc_Opaque)0);
     Rpc_ServerCreate(udpSocket, (Rpc_Proc)CUSTOMS_JOBS, ImportInfo,
