@@ -17,7 +17,7 @@
  */
 #ifndef lint
 static char *rcsid =
-"$Id: mca.c,v 1.39 1999/08/22 23:15:10 stolcke Exp $ ICSI (Berkeley)";
+"$Id: mca.c,v 1.42 2001/05/08 00:02:42 stolcke Exp $ ICSI (Berkeley)";
 #endif /* not lint */
 
 #include    <netdb.h>
@@ -109,7 +109,7 @@ MCAFindHostAddr (addr, name, create)
     LstNode 	  	ln;
     ServerPtr	  	servPtr;
 
-    if (addr->sin_addr.s_addr == htonl(INADDR_LOOPBACK)) {
+    if (addr->sin_addr.s_addr == htonl(INADDR_LOOPBACK) && us != NULL) {
 	/*
 	 * 'localhost' address (127.1) means us.
 	 */
@@ -409,17 +409,10 @@ MCAAvail (from, msg, len, avail)
 			avail->rating);
 	    }
 	    /*
-	     * Record new availability, preserving the ALLOC and EXCLUSIVE flags
+	     * Record new availability, preserving the ALLOC flag
 	     */
-	    hostPtr->avail = (hostPtr->avail & (AVAIL_ALLOC|AVAIL_EXCLUSIVE))
-			      | avail->avail;
+	    hostPtr->avail = (hostPtr->avail & AVAIL_ALLOC) | avail->avail;
 	    hostPtr->rating = avail->rating;
-	    /*
-	     * Exclusive use terminates as soon as all jobs are gone.
-	     */
-	    if (hostPtr->nJobs == 0) {
-		hostPtr->avail &= ~AVAIL_EXCLUSIVE;
-	    }
 	}
 
 	interval.tv_sec = avail->interval.tv_sec;
@@ -496,8 +489,6 @@ MCAAvailHost (servPtr, data)
 	 (servPtr != data->clntPtr)) &&	    /* ... or the client is not local */
 	(!(data->flags & EXPORT_SAME) ||    /* arch need not be same */
 	 (servPtr->arch == data->clntPtr->arch)) &&	/* ... or it is */
-	(!(data->flags & EXPORT_EXCLUSIVE) ||  /* no exclusive use needed */
-	 (servPtr->nJobs == 0)) &&	     /* ... machine is empty */
 	(!(data->flags & EXPORT_68020) || (servPtr->arch <= 2)) && /*XXX*/
 	((servPtr->clients == NILLST) ||    /* host accepts all comers */
 	 (Lst_Member (servPtr->clients,	    /* ... or at least this one */
@@ -540,7 +531,7 @@ MCA_HostInt (from, msg, len, data)
     ServerPtr	  	clntPtr;    /* Client requesting Host */
     ServerPtr	  	servPtr;    /* Agent to serve client */
     ExportPermit	permit;	    /* Permit to send to client */
-    ExportPermit  	allocPermit;/* Permit to send to server */
+    AllocData  		allocData;  /* Data to send to server */
     int	    	  	uid;
 
     if (len < sizeof(Host_Data)) {
@@ -627,13 +618,17 @@ MCA_HostInt (from, msg, len, data)
 		/*
 		 * Before we reply to the requesting server, we must tell the
 		 * victim what a lucky machine it is...
+		 * Pass the EXPORT_* flags from the client to help the
+		 * victim decide whether to accept the challenge.
 		 */
 		if (clntPtr == us) {
-		    allocPermit.addr.s_addr = CUSTOMS_FROMADDR;
+		    allocData.permit.addr.s_addr = CUSTOMS_FROMADDR;
 		} else {
-		    allocPermit.addr = clntPtr->addr;
+		    allocData.permit.addr = clntPtr->addr;
 		}
-		allocPermit.id = permit.id;
+		allocData.permit.id = permit.id;
+		allocData.flags = data->flags;
+
 		victim.sin_family = AF_INET;
 		victim.sin_port = htons(udpPort);
 		victim.sin_addr = servPtr->addr;
@@ -643,20 +638,13 @@ MCA_HostInt (from, msg, len, data)
 		 */
 		servPtr->avail |= AVAIL_ALLOC;
 		stat = Rpc_Call(udpSocket, &victim, (Rpc_Proc)CUSTOMS_ALLOC,
-				sizeof(allocPermit),
-				(Rpc_Opaque)&allocPermit,
+				sizeof(allocData),
+				(Rpc_Opaque)&allocData,
 				sizeof(reply),
 				(Rpc_Opaque)&reply,
 				CUSTOMSINT_NRETRY, &retryTimeOut);
 		if (stat == RPC_SUCCESS) {
 		    servPtr->avail = reply.avail;
-		    /*
-		     * If the user requested exclusive use and got it,
-		     * well, it's his for the time being.
-		     */
-		    if (data->flags & EXPORT_EXCLUSIVE) {
-			servPtr->avail |= AVAIL_EXCLUSIVE;
-		    }
 		    servPtr->rating = reply.rating;
 		    lastAlloc = servPtr;
 		    break;
@@ -671,6 +659,7 @@ MCA_HostInt (from, msg, len, data)
 		    }
 		    servPtr->avail = AVAIL_DOWN;
 		    servPtr->rating = time((time_t *)0);
+	    	    servPtr->nJobs = 0;
 		} else {
 		    /*
 		     * Outright rejection.  Maybe due to access restrictions
