@@ -59,7 +59,7 @@
  *	Arch_Init 	    	Initialize this module.
  */
 #ifndef lint
-static char *rcsid = "$Id: arch.c,v 1.29 1995/03/11 00:23:36 stolcke Exp $ ICSI (Berkeley)";
+static char *rcsid = "$Id: arch.c,v 1.30 1999/09/16 13:57:33 stolcke Exp $ ICSI (Berkeley)";
 #endif /* not lint */
 
 #include    <stdio.h>
@@ -115,20 +115,32 @@ typedef struct Arch {
 #define ARCH_COMMON	0x01      /* SYSV Common Archive format */
 #define ARCH_LONGNAM	0x02      /* SVR4-style long name indirection */
 #define ARCH_NEWBSD	0x04      /* 4.4BSD style long name extension */
+#define ARCH_GNU	0x08      /* GNU style long name indirection */
 #define ARCH_BAD	0x80      /* Format error detected by ArchScan() */
 } Arch;
 
 #define ARCH_TOC	"/"   /* Format-independent hash key for the table
 			       * of contents member (a string that cannot
 			       * be a real filename). */
+#define ARCH_ISTOC(x)	(strncmp(x, LIBTOC, sizeof(LIBTOC)-1) == 0 && \
+                   	 (x)[sizeof(LIBTOC)-1] == ' ')
+			      /* ranlib archive TOC 
+			       * Note: the above test handles NeXT's
+			       * `__.SYMDEF SORTED' (and possibly other
+			       * extensions to `__.SYMDEF'.
+			       */
 #define ARCH_ISCOMTOC(x) (strncmp(x, "/ ", 2)==0)
 			      /* Common archive TOC */
-#define ARCH_ISCOMNAMES(x) (strncmp(x, "// ", 2)==0)
+#define ARCH_ISCOMNAMES(x) (strncmp(x, "//", 2)==0)
 			      /* Common archive long name section */
+#define ARCH_ISGNUNAMES(x) (strncmp(x, "ARFILENAMES/", 12)==0)
+			      /* GNU binutils long name section */
 #define ARCH_ISLONGNAME(x) (x[0] == '/' && isdigit(x[1]))
 			      /* Common archive long name reference */
 #define ARCH_ISBSDLONGNAME(x) (strncmp(x, "#1/", 3)==0)
 			      /* 4.4BSD style long name reference */
+#define ARCH_ISGNULONGNAME(x) (x[0] == ' ' && isdigit(x[1]))
+			      /* GNU binutils long name reference */
 
 static FILE *ArchFindMember();
 
@@ -192,7 +204,9 @@ again:
 	    }
 	    *typePtr = ARCH_BAD;
 	    return (NULL);
-	} else if (ARCH_ISCOMNAMES(arhPtr->ar_name)) {
+	} else if (ARCH_ISCOMNAMES(arhPtr->ar_name) ||
+		   ARCH_ISGNUNAMES(arhPtr->ar_name))
+	{
 	    /*
 	     * Read the long names section into memory for later reference
 	     */
@@ -211,10 +225,16 @@ again:
 	    if (DEBUG(ARCH)) {
 		Debug("long names (%d bytes)...", longSize);
 	    }
-	    *typePtr |= ARCH_COMMON;
+	    if (ARCH_ISCOMNAMES(arhPtr->ar_name)) {
+		    *typePtr |= ARCH_COMMON;
+	    } else {
+		    *typePtr |= ARCH_GNU;
+	    }
 	    *typePtr |= ARCH_LONGNAM;
 	    goto again;
-	} else if (ARCH_ISLONGNAME(arhPtr->ar_name)) {
+	} else if (ARCH_ISLONGNAME(arhPtr->ar_name) ||
+		   ARCH_ISGNULONGNAME(arhPtr->ar_name))
+	{
 	    char *cp;
 	    int idx = atoi(arhPtr->ar_name + 1);
 
@@ -232,7 +252,8 @@ again:
 	     * already been replaced by \0.
 	     */
 	    for (cp = longNames + idx;
-	         (cp < longNames + longSize) && *cp && (*cp != '/');
+	         (cp < longNames + longSize) && *cp &&
+			(*cp != ((*typePtr & ARCH_GNU) ? '\n' : '/'));
 		 cp++);
 	    if (cp == longNames + longSize) {
 		if (DEBUG(ARCH)) {
@@ -257,13 +278,7 @@ again:
 	    }
 	    memName = ARCH_TOC;
 	    *typePtr |= ARCH_COMMON;
-	} else if (strncmp(arhPtr->ar_name, LIBTOC, sizeof(LIBTOC)-1) == 0 &&
-	           arhPtr->ar_name[sizeof(LIBTOC)-1] == ' ')
-	{
-	    /*
-	     * Note: the above test handles NeXT's `__.SYMDEF SORTED'
-	     * (and possibly other extensions to `__.SYMDEF'.
-	     */
+	} else if (ARCH_ISTOC(arhPtr->ar_name)) {
 	    if (DEBUG(ARCH)) {
 		Debug("%s t.o.c...", LIBTOC);
 	    }
@@ -439,7 +454,6 @@ Arch_FindSymbol (archive, symbol)
 		free (offsets);
 		goto cleanup;
 	    }
-	    nSymbols = ntohl(nSymbols);
 
 	    strTableLen = tocSize - sizeof (nSymbols) * (nSymbols + 1);
 	    strTable = emalloc(strTableLen);
@@ -531,16 +545,15 @@ Arch_FindSymbol (archive, symbol)
 	 * We'd like to use ArchScan to get the member name for this offset.
 	 * To resolve any long name indirections we have to scan the long names
 	 * section as well.  We assume it follows the TOC, if at all present.
-	 * Of course, this is needed only with Common Archive format.
 	 */
-	if (ARCH_ISCOMTOC(arh.ar_name) && 
-	    (offset != ((tocPos + tocSize + 1)&~1L)) &&
+	if ((offset != ((tocPos + tocSize + 1)&~1L)) &&
 	    (fseek (arch, (tocPos + tocSize + 1)&~1L, 0) >= 0))
 	{
 	    (void)ArchScan(arch, &arh, &arType);
 	}
+
 	/*
-	 * No we can seek to the previously determined offset and scan that
+	 * Now we can seek to the previously determined offset and scan that
 	 * member, returning us the member name.
 	 */
 	if (fseek (arch, offset, 0) < 0 ||
